/* Petal: An open source system for classification learning from very large data
** Copyright (C) 2012 Geoffrey I Webb
**
** Module for registering each of the available learners.
** WOuld ideally use a map from char* to learner constructors, but c++ does not seem to support that
**
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program. If not, see <http://www.gnu.org/licenses/>.
**
** Please report any bugs to Geoff Webb <geoff.webb@monash.edu>
*/
#include "learnerRegistry.h"
#include "utils.h"


/// this access method should be used to access the learner registry during initialisation so as to ensure that the object is initialised before used
LearnerRegistry& getLearnerRegistry() {
static LearnerRegistry theLearnerRegistry;
  return theLearnerRegistry;
}

learner* LearnerRegistry::createLearner(const char *name, char*const*& argv, char*const* end) {
  std::map<std::string, PtrToLearnerConstructor>::const_iterator it;

  it = constructors_.find(name);
  
  if (it != constructors_.end()) {
    return it->second(argv, end);
  }

  return NULL;
}

void LearnerRegistry::registerLearner(const char* name, PtrToLearnerConstructor constructor) {
  constructors_[name] = constructor;
}

void LearnerRegistry::getLearnerList(std::vector<std::string>& learners) const {
  for (std::map<std::string, PtrToLearnerConstructor>::const_iterator it = constructors_.begin(); it != constructors_.end(); it++) {
    learners.push_back(it->first);
  }
}

/// create an object of this type in order to register a new learner
LearnerRegistrar::LearnerRegistrar(const char* name, PtrToLearnerConstructor constructor) {
  getLearnerRegistry().registerLearner(name, constructor);
}
